import {
  csv,
  select,
  geoIdentity,
  geoPath,
  geoCentroid,
  geoEqualEarth,
  geoConicConformal,
  geoOrthographic,
  geoAlbers,
  geoNaturalEarth1,
  zoom,
  event,
  scaleOrdinal,
  schemeSpectral,
  scaleLinear,
  scaleThreshold,
  max,
  format,
  json,
  geoMercator,
  Legend,
  legendColor,
} from 'd3';
import { feature } from 'topojson';

const titleText =
  'Map of top-rated restaurants in Greater Boston';

const svg = select('svg').attr("class", "legend");

const width = +svg.attr('width');
const height = +svg.attr('height');

const projection = geoIdentity();
const pathGenerator = d3.geoPath().projection(projection);

var g = svg.append('g');

svg.call(
  zoom().on('zoom', () => {
    g.attr('transform', event.transform);
  })
);


  const colorScale = scaleThreshold()
    .domain([10, 50, 100, 1000])
    .range([
      '#fde3dc',
      '#eea08c',
      '#dc7057',
      '#a7422f',
      '#890101',
    ]);

Promise.all([
  csv('data.csv'),
  json(
    'https://gist.githubusercontent.com/bycoffe/5871252/raw/368cba4746f8d57759fb7efcbaaa3daf4611620b/ma-towns.topojson'
  ),
]).then(([yelpData, topoJSONdata]) => {
  const towns = feature(
    topoJSONdata,
    topoJSONdata.objects.towns
  );

  const countByCity = new Map(
    yelpData.map((d) => [d.city, +d.count])
  );

  towns.features.forEach((d) => {
    d.properties.count = countByCity.get(d.properties.town);
  });

  const tooltip = d3
    .select('body')
    .append('div')
    .attr('class', 'd3-tooltip')
    .style('position', 'absolute')
    .style('z-index', '10')
    .style('visibility', 'hidden')
    .style('padding', '10px')
    .style('background', 'rgba(0,0,0,0.6)')
    .style('border-radius', '5px')
    .style('color', 'white');


  g.selectAll('path')
    .data(towns.features)
    .enter()
    .append('path')
    .attr('class', 'towns')
    .attr('d', (d) => pathGenerator(d))
    .attr('fill', (d) =>
      d.properties.count
        ? colorScale(d.properties.count)
        : '#e0e0e0'
    )
    .on('mouseover', function (d, i) {
    console.log(d.properties.count)
      if (d.properties.count) {
        tooltip
          .html(
            `${d.properties.town} : ${d.properties.count} great restaurants`
          )
          .style('visibility', 'visible');
      } else {
        tooltip
          .html(`${d.properties.town} : no data`)
          .style('visibility', 'visible');
      }
    })
    .on('mousemove', function () {
      tooltip
        .style('top', event.pageY - 10 + 'px')
        .style('left', event.pageX + 10 + 'px');
    })
    .on('mouseout', function () {
      tooltip.html(``).style('visibility', 'hidden');
    });

  g.append('text')
    .attr('class', 'title')
    .attr('x', 7)
    .attr('y', 300)
    .text(titleText);
  
  const legendProps = {
  width: '10em',
  height: '1.2em',
  yAxisLabel: 'Number of restaurants'
}  
 const linearGradient = g.append('linearGradient')
    .attr('id', 'linearGradient')
    // Define vertical gradient
    .attr('x1', '0%').attr('x2', '100%')
    .attr('y1', '0%').attr('y2', '0%');

  linearGradient.selectAll('stop')
    .data( colorScale.range() )
    .enter().append('stop')
    .attr('offset', function(d,i) { 
			 return i/(colorScale.range().length-1);
		})
    .attr('stop-color', function(d) { return d; });

g.append('rect')
  .attr('id', 'legend')
    .attr('width', legendProps.width)
        .attr('height', legendProps.height)
       .attr('x', '7')
  		  .attr('y', '360')
        .style('fill', 'url(#linearGradient)');
  
  g.append('text')
  		.attr('class', 'legend-label')
    	.attr('x', '7')
  		.attr('y', '400')
      .text(legendProps.yAxisLabel);   
  
});


